#
#	This package uses the methods of Clauset, Shalizi and Newman (2009)
#	to test whether we have a power-law with greater refinement.
#
#	https://stats.stackexchange.com/questions/108843/how-to-test-whether-a-distribution-follows-a-power-law
#-------------------------------------------------------------------
library(poweRlaw)
library(methods)
#
#	Bring in interp1.
#
library(pracma)

args 	<- commandArgs(trailingOnly = TRUE)
#
#	For this dataset, its small enough to use the whole thing without exponential
#	binning and is already a rank-ordered ccdf.
#
indexOfLastNonZero <- function(x) {
	ix	= 0
	for(i in 1:length(x)) {
		ix = i
        	if (x[i] == 0) {
			ix = i
               break
        	}
     } 

	return(ix-1)
}
#
#	Read in the frequency data.
#
data <- read.table("transfer_ccdf_clauset.txt",header=FALSE)
x    <- data$V1
y    <- data$V2
#
#	The poweRlaw package just takes a vector but it must be as a rank order so evenly
#	sampled in x so we resample it.
#
#	Make sure the xr vector does not exceed the data range x.
#
xr	<- c(seq(2,100,1))
yr	<- interp1(x, y, xi = xr)
yt	<- yr[1:indexOfLastNonZero(yr)]
#
#	Plot as sanity check.
#
#xt	<- xr[1:length(yt)]
#plot(xt,yt,log="xy")

plobj	<- conpl$new(yt)
#
#	Estimate the xmin and the exponent α of the power law, and assign them to the power law object
#
est <- estimate_xmin(plobj,xmax=1e+07)
plobj$xmin <- est$xmin
plobj$pars <- est$pars
print("Power-law parameters, xmin, pars(alpha)")
plobj$xmin
plobj$pars
#
#	Also, at this point, you can see the KS statistic.
#
print("KS statistic, (max distance between data and power-law distribution")
est$KS
#
#	KS statistic tells you how well power law distribution fits your data, but it 
#	doesn't tell you how likely your data is drawn from power law. So you also need a p value. 
#	This is how you do it:  (Note, no of sims = 1000 requires several hours).
#
print("Estimated p-value.  Should be bigger than 0.1 for plausibility.")
bs <- bootstrap_p(plobj,xmax=1e+07,no_of_sims=10)
bs$p
